import os

from mistralai.client import MistralClient

from utils_ai.generic_ai import GenericAI


class MistralAIProvider(GenericAI):
    NAME = 'MistralAI'
    MODEL = 'mistral-large-latest'

    def __init__(self):
        GenericAI.__init__(self)

        self.client = MistralClient(api_key=os.getenv('MISTRAL_API_KEY'))

    def get_chat_reply(self, messages: list) -> str:
        response = self.client.chat(
            model=MistralAIProvider.MODEL, messages=messages
        )
        return response.choices[0].message.content



from utils_ai.generic_ai.GenericAIImage import GenericAIImage
from utils_ai.generic_ai.GenericAIText import GenericAIText


class GenericAI(
    GenericAIText,
    GenericAIImage,
):
    def __init__(self):
        GenericAIText.__init__(self)
        GenericAIImage.__init__(self)



import random

from utils_ai.providers.MistralAIProvider import MistralAIProvider
from utils_ai.providers.OpenAIProvider import OpenAIProvider


class ProviderFactory:
    PROVIDER_LIST = [
        OpenAIProvider,
        MistralAIProvider,
    ]

    @staticmethod
    def random():
        provider = random.choice(ProviderFactory.PROVIDER_LIST)
        return provider()

    @staticmethod
    def from_name(name: str):
        if name == OpenAIProvider.NAME:
            return OpenAIProvider()
        if name == MistralAIProvider.NAME:
            return MistralAIProvider()
        raise ValueError(f'Unknown provider name: {name}')



import os
import sys

from utils_base import Console, File, Log

from utils_ai.providers import ProviderFactory

log = Log('ChatApp')


class ChatApp:
    @staticmethod
    def set_profile(ai, profile_path: str):
        if os.path.exists(profile_path):
            profile_content = File(profile_path).read()
            ai.set_profile(profile_content)
            print(Console.note(f'{profile_path}'))
            print('')

    @staticmethod
    def run_iter(ai):
        message = input('>> ')
        print('')

        if len(message) == 0:
            return

        if message in ['quit', 'exit', 'q', 'x', 'exit()', 'quit()']:
            print(Console.note('Bye!'))
            print()
            sys.exit(0)

        if message.lower().startswith('draw:'):
            image_path = ai.draw(prompt=message[5:])
            if image_path:
                print(Console.note(image_path))
                print('')
                os.startfile(image_path)
            return

        reply = ai.chat(message)
        if reply:
            print(Console.note(reply))
            print('')

    @staticmethod
    def run(provider_name: str, profile_path: str):
        ai = ProviderFactory.from_name(provider_name)
        print('')
        print(Console.note(f'[{ai.NAME}/{ai.MODEL}]'))
        print('')

        ChatApp.set_profile(ai, profile_path)

        while True:
            ChatApp.run_iter(ai)



from attr import dataclass

from utils_ai.core.ChatRole import ChatRole


@dataclass
class Message:
    role: ChatRole
    content: str

    def todict(self):
        return dict(role=self.role, content=self.content)



from utils_base import Log

from utils_ai.core.ChatRole import ChatRole
from utils_ai.core.Message import Message

log = Log('GenericAIText')


class GenericAIText:
    def __init__(self):
        self.messages = []

    def set_profile(self, profile_content: str):
        self.messages.append(
            Message(role=ChatRole.system, content=profile_content).todict()
        )

    def append_message(self, role: ChatRole, content: str):
        self.messages.append(Message(role=role, content=content).todict())

    def get_chat_reply(self, messages: list) -> str:
        raise NotImplementedError

    def chat(self, message: str) -> str:
        self.append_message(ChatRole.user, message)
        try:
            reply = self.get_chat_reply(self.messages)
            self.append_message(ChatRole.assistant, reply)
            return reply
        except Exception as e:
            log.error(str(e))
            return None



# utils_ai.generic_ai (auto generate by build_inits.py)

from utils_ai.generic_ai.GenericAI import GenericAI
from utils_ai.generic_ai.GenericAIImage import GenericAIImage
from utils_ai.generic_ai.GenericAIText import GenericAIText



# utils_ai (auto generate by build_inits.py)

from utils_ai.apps import ChatApp
from utils_ai.core import ChatRole, Message
from utils_ai.generic_ai import GenericAI, GenericAIImage, GenericAIText
from utils_ai.providers import (MistralAIProvider, OpenAIProvider,
                                ProviderFactory)



import os
import tempfile

from utils_base import Log, Hash
from utils_www import WWW

log = Log('GenericAIImage')


class GenericAIImage:
    DIR_DESKTOP = os.environ['DIR_DESKTOP'] 
    def get_image_url(self, prompt: str) -> str:
        raise NotImplementedError

    def draw(self, prompt: str):
        h = Hash.md5(prompt)[:8]
        image_path = os.path.join(GenericAIImage.DIR_DESKTOP, f'{h}.png')
        if os.path.exists(image_path):
            log.warn(f'Image already exists.')
            return image_path
            
        try:
            image_url = self.get_image_url(prompt)
            WWW.download_binary(image_url, image_path)
            return image_path
        except Exception as e:
            log.error(str(e))
            return None



# utils_ai.providers (auto generate by build_inits.py)

from utils_ai.providers.MistralAIProvider import MistralAIProvider
from utils_ai.providers.OpenAIProvider import OpenAIProvider
from utils_ai.providers.ProviderFactory import ProviderFactory



import os

from openai import OpenAI

from utils_ai.generic_ai import GenericAI


class OpenAIProvider(GenericAI):
    NAME = 'OpenAI'
    MODEL = 'gpt-4'
    IMAGE_MODEL = 'dall-e-3'

    def __init__(self):
        GenericAI.__init__(self)

        self.client = OpenAI(api_key=os.getenv('OPENAI_API_KEY'))

    def get_chat_reply(self, messages: list) -> str:
        response = self.client.chat.completions.create(
            model=OpenAIProvider.MODEL, messages=messages
        )
        return response.choices[0].message.content

    def get_image_url(self, prompt: str) -> str:
        response = self.client.images.generate(
            model=OpenAIProvider.IMAGE_MODEL,
            prompt=prompt,
            size="1024x1024",
            quality="standard",
            n=1,
        )
        return response.data[0].url
